'use strict'
############################################################
#
# 이중상자수염그림
#
#
do ()->
  d3.svg.bibox = ()->
    scaleX = (d)-> d
    valueX = (d)-> d.x
    scaleY = (d)-> d
    valueY = (d)-> d.y
    whiskerWidth = 30

    # 최대・최소의 수염을 그리기
    whisker = (source, target, isX)->
      ()->
        s = this.append('g').classed('whisker', true)
        mainPath = "M#{source[0]},#{source[1]}L#{target[0]},#{target[1]}"
        whiskerPath = "M#{source[0] - whiskerWidth / 2},#{target[1]}L#{source[0] + whiskerWidth / 2},#{target[1]}"
        if isX
          whiskerPath = "M#{target[0]},#{target[1] - whiskerWidth / 2}L#{target[0]},#{target[1] + whiskerWidth / 2}"
        s.append('path').attr('d', mainPath)
        s.append('path').attr('d', whiskerPath)

    # 다섯 개의 값(중앙값, 최댓값, 최솟값, 1사분위, 3사분위)을 계산한다
    getFive = (data)->
      max: Math.max.apply Math, data
      min: Math.min.apply Math, data
      first: d3.quantile data, 0.25
      third: d3.quantile data, 0.75
      median: d3.median data

    # 스케일을 적용하면 다섯 개의 값을 계산한다
    getScaledFive = (five, scale)->
      iqr = five.third - five.first
      min = scale(five.min)
      mim = scale(five.first - 1.5 * iqr) if five.min < five.first - 1.5 * iqr
      max = scale(five.max)
      max = scale(five.third + 1.5 * iqr) if five.max > five.third + 1.5 * iqr

      min: min
      max: max
      median: scale(five.median)
      first: scale(five.first)
      third: scale(five.third)

    # box본체
    box = (g)->
      g.each ()->
        g = d3.select(this)
        # 데이터를 가져오기
        data = this.__data__

        # x축, y축 값을 가져오기
        arrX = data.map(valueX).filter((d)-> not isNaN d).sort((a, b)-> a - b)
        arrY = data.map(valueY).filter((d)-> not isNaN d).sort((a, b)-> a - b)

        # 다섯 개 값을 계산
        fiveX = getFive(arrX)
        fiveY = getFive(arrY)

        # Interquartile Range를 계산
        iqrX = fiveX.third - fiveX.first
        iqrY = fiveY.third - fiveY.first

        # 특잇값 구하기 
        outlier = data.filter((d)->
          not ((fiveX.first - 1.5 * iqrX) < valueX(d) < (fiveX.third + 1.5 * iqrX) and
            (fiveY.first - 1.5 * iqrY) < valueY(d) < (fiveY.third + 1.5 * iqrY)))

        # 스케일된 다섯 개 요소 값을 계산
        scaledX = getScaledFive(fiveX, scaleX)
        scaledY = getScaledFive(fiveY, scaleY)

        # 폭과 높이 계산
        width  = Math.abs scaledX.third - scaledX.first
        height = Math.abs scaledY.third - scaledY.first

        # rect요소 그리기
        rect = g.append('rect').attr(
          width: width, height: height, x: scaledX.first, y: scaledY.third, fill: 'none')

        # 중앙값을 그리기
        medianLineX = g.append('line').classed('median', true).attr(
          x1: scaledX.first, x2: scaledX.first + width
          y1: scaledY.median, y2: scaledY.median)
        medianLineY = g.append('line').classed('median', true).attr(
          x1: scaledX.median, x2: scaledX.median
          y1: scaledY.third, y2: scaledY.third + height)

        # 최댓값・최솟값의 수염을 그리기
        g.call(whisker(
          [scaledX.first + width / 2, scaledY.first],
          [scaledX.first + width / 2, scaledY.min]))
        g.call(whisker(
          [scaledX.first + width / 2, scaledY.third],
          [scaledX.first + width / 2, scaledY.max]))
        g.call(whisker(
          [scaledX.first, scaledY.third + height / 2],
          [scaledX.min, scaledY.third + height / 2], true))
        g.call(whisker(
          [scaledX.third, scaledY.third + height / 2],
          [scaledX.max, scaledY.third + height / 2], true))

        # 특잇값 그리기
        g.selectAll('circle').data(outlier).enter().append('circle').attr(
          cx: ((d)-> scaleX(valueX(d)))
          cy: ((d)-> scaleY(valueY(d)))
          r: 3, fill: 'none'
        )

    # 메소드를 설정
    box.x = (_scale)->
      return scaleX unless _scale
      scaleX = _scale
      box
    box.y = (_scale)->
      return scaleY unless _scale
      scaleY = _scale
      box
    box.valueX = (_value)->
      return valueX unless _value
      valueX = _value
      box
    box.valueY = (_value)->
      return valueY unless _value
      valueY = _value
      box
    box.whiskerWidth = (_width)->
      return whiskerWidth unless _width
      whiskerWidth = _width
      box
    box
