'use strict'
# 그리기 영역 정보 
margin = new Margin(50, 100)
[width, height] = [600, 400]

#------------------------------------------------------------
#
# ## changeBubbleOpacityByDay
#
# 특정 일의 꺾은선그래프를 마우스 오버 / 마우스 아웃했을 때의 핸들러 
# 대응 일짜의 버블차트의 버블의 투명도만 1로 그 외에는 0.1로 한다. 
#
# * @param  day         오브젝트   꺾은선그래프에서 마우스오버된 날짜 데이터 
# * @param  bubbleChart d3.selection 버블차트의 svg요소
# * @return null
#
changeBubbleOpacityByDay = (day, bubbleChart = d3.select('#bounce_and_elapse'))->
  if day?
    bubbleChart.selectAll('g.day')
      .style('opacity', 0.1)
    bubbleChart.selectAll("g.day.d#{new Date(day.key).getTime()}")
      .style('opacity', (d)-> 1.0)
  else
    bubbleChart.selectAll('g.day')
      .style('opacity', 1.0)

#------------------------------------------------------------
#
# ## drawScatterPlot
#
# 버블차트 작성함수 
#
# * @param  target       문자열 버블차트를 추가하는 요소의 XPah
# * @param  data         배열    데이터 배열 
# * @return d3.selection 버블차트의 svg요소 
#
drawScatterPlot = (target, data)->
  # svg요소를 작성 
  svg = d3.select(target).append("svg")
    .attr(
      width: width + margin.width
      height: height + margin.height
    )
  main = svg
    .append("g").attr(
      width: width
      height: height
      transform: "translate(#{margin.left},#{margin.top})"
    )

  # x축 y축의 최댓값을 가져온다 
  xmax = d3.max(data.map((d)-> d3.max(d.values, (e)-> e.values.elapse)))
  ymax = d3.max(data.map((d)-> d3.max(d.values, (e)->
    if +e.key is 19 then 0 else e.values.bounce / e.values.visitor)))
  rmax = d3.max(data.map((d)-> d3.max(d.values, (e)->
    if +e.key is 19 then 0 else e.values.visitor)))

  # x축, y축의 반지름, 색의 스케일 
  x = d3.scale.linear().range([0, width]).domain([0, xmax])
  y = d3.scale.linear().range([height, 0]).domain([0, ymax])
  r = d3.scale.linear().range([1, 100]).domain([0, rmax])
  color = d3.scale.category20()

  # 일자별 g요소를 작성 
  day = main.selectAll('g.day').data(data).enter().append('g')
    .attr('class', (d)-> "day d#{new Date(d.key).getTime()}")

  # 버블의 g요소를 작성
  bubble = day.selectAll('g.bubble').data((d)-> d.values).enter()
    .append('g').attr(
      class: 'bubble'
      transform: (d)-> "translate(#{x(d.values.elapse)},#{y(d.values.bounce / d.values.visitor)})"
    )
  bubble.append('circle')
    .attr(
      r: (d)-> Math.sqrt(r(d.values.visitor))
      fill: (d)-> if +d.key is 19 then 'none' else color(d.key)
      stroke: 'grey')
    .style('opacity', 0.8)
  bubble.append('text').text((d)-> d.key)
    .attr(dy: 3).style('text-anchor', 'middle')

  # x축 y축 작성 
  xaxis = d3.svg.axis().scale(x).tickFormat((d)-> "#{d}초")
  yaxis = d3.svg.axis().orient("left").scale(y).tickFormat(d3.format("%"))
  xAxis = main.append('g').attr('class', 'axis').call(xaxis)
    .attr('transform', "translate(0, #{height})")
  yAxis = main.append('g').attr('class', 'axis').call(yaxis)

  # x축 y축 타이틀 작성 
  xtitle = main.append('text').text('스테이지 플레이시간')
    .attr('transform', "translate(#{width / 2}, #{height + 40})")
  ytitle = main.append('text').text('이탈률')
    .attr('transform', "rotate(-90) translate(#{- height / 2}, #{-60})")

  # 버블 크기의 범례 작성 
  raxis = d3.svg.axis().scale(r)
  legend = main.append('g')
  legend.selectAll('circle').data(d3.range(0, 0|rmax, (0|rmax) / 3)).enter().append('circle')
    .attr(
      r: (d)-> Math.sqrt(r(d))
      cx: (d, idx)-> 30
      cy: (d, idx)-> 20 * idx
      fill: 'none'
      stroke: 'grey')
  legend.selectAll('text').data(d3.range(0, 0|rmax, (0|rmax) / 3)).enter().append('text')
    .text((d)-> (0|d)+"명")
    .attr(
      dx: 50
      dy: (d, idx)-> 20 * idx + 5
      fill: 'none'
      stroke: 'grey')

  main

#------------------------------------------------------------
#
# ## drawSurviveLineChart
#
# 일자별 잔존률의 꺾은선그래프를 작성하는 함수 
#
# * @param  target        문자열        svg를 추가하는 요소의 XPath
# * @param  data          배열         데이터의 배열 
# * @param  bubbleChart   d3.selection 버블차트의 svg요소 
#
drawSurviveLineChart = (target, data, bubbleChart)->
  # svg요소의 작성 
  svg = d3.select(target).append("svg")
    .attr(
      width: width + margin.width
      height: height + margin.height)
    .append("g").attr(
      width: width
      height: height
      transform: "translate(" + margin.left + "," + margin.top + ")")

  # x축, y축의 범위 작성 
  xExtent = d3.extent data[0].values, (d)-> +d.key
  yExtent = d3.extent data[0].values, (d)-> d.surviveRate
  # x축, y축의 스케일 작성 
  x = d3.scale.linear().domain(xExtent).range([0, width])
  y = d3.scale.linear().range([height, 0])

  # 선분군의 path를 작성하는 함수
  line = d3.svg.line().x((d)-> x(+d.key))
  # 투명도의 스케일을 작성하는 
  opacities = d3.time.scale().domain([new Date(data[0].key), new Date()]).range([0, 1])

  # 일짜별로 꺾은선을 그린다 
  data.forEach (day, idx)->
    initVisitor = day.values[0].values.visitor
    line.y((d)-> y(d.values.visitor / initVisitor))
    path = svg.datum(day.values)
      .append('path')
      .attr(
        d: line
        fill: 'none'
        stroke: 'grey'
        'stroke-width': 3)
      .style('opacity', opacities(new Date(day.key)))
    # 꺾은선을 마우스오버, 마우스아웃했을 때 버블차트를 하이라이트하는 이벤트 추가 
    if bubbleChart
      path
        .on('mouseover', ()-> changeBubbleOpacityByDay(day, bubbleChart))
        .on('mouseout', ()-> changeBubbleOpacityByDay(null, bubbleChart))

  # x축, y축을 작성
  xaxis = d3.svg.axis().scale(x)
  yaxis = d3.svg.axis().scale(y).orient('left').tickFormat(d3.format('%'))
  xAxis = svg.append('g').classed('axis', true).call(xaxis).attr('transform', "translate(0,#{height})")
  yAxis = svg.append('g').classed('axis', true).call(yaxis)

  # x축, y축의 타이틀을 작성
  xtitle = svg.append('text').text('스테이지 번호').attr('transform', "translate(#{width / 2}, #{height + 40})")
  ytitle = svg.append('text').text('잔존률').attr('transform', "rotate(-90) translate(#{- height / 2}, #{-60})")


#------------------------------------------------------------
#
# 샘플 튜토리얼의 액세스 로그 데이터를 작성하고 
# 날짜별 스테이지 돌입 유저수를 꺾은선 그래프로
# 각 스테이지 클리어에 걸린 시간과 이탈률을 버블차트로 
# 표현하는 시각화를 작성
#

# 로그데이터 수 
recordNum = 10000
# 게임 샘플로그 생성 
gameLog = getSampleData(recordNum)

# 유저Id로 정리하여 세션으로 변환(대수처리)
users = d3.nest()
  .key((d)-> d.userId)
  .map(gameLog)

# 사용자별 세션에 스테이지 처리시간과 이탈 플래그를 추가 
for userId, session of users
  for stage, sid in session
    if session[sid + 1]?
      stage.elapse = (new Date(session[sid + 1].time) - new Date(stage.time)) / 1000
      stage.bounce = 0
    else
      stage.bounce = 1
      stage.elapse = 0

# 일자별・스테이지별로 정리(대수・통계처리)
data = d3.nest()
  .key((d)-> d3.time.day(d.time))
  .sortKeys((a, b)-> new Date(b).getTime() - new Date(a).getTime())
  .key((d)-> d.stageId)
  .rollup((vals)->
    {
      visitor: vals.length
      elapse: d3.mean(vals.filter((d)-> d.elapse > 0), (d)-> d.elapse) or 0
      bounce: d3.sum(vals, (d)-> d.bounce)
    }
    )
  .entries(gameLog)

# 산포도그리기 
bubbleChart = drawScatterPlot("#bounce_and_elapse", data)
# 생존곡선그리기 
drawSurviveLineChart("#bounce_by_data", data, bubbleChart)
