'use strict'
# 그리기 영역 정보 
margin = new Margin(50)
[width, height] = [1000, 500]

# 애니메이션시간
duration = 500

#------------------------------------------------------------
#
# ## normalize
#
# 합계가1.0이 되도록 정규화한 값을 nVal에 추가한다 
#
# * @param  values  배열 
#
normalize = (values)->
  max = d3.max values, (d)-> d.value
  for value in values
    value.nVal = value.value / max
  values

#------------------------------------------------------------
#
# ## drawOpenSpending
#
# 세금은어디로갔나 방식 시각화 작성함수 
#
# * @param  target 문자열 시각화를추가할요소의XPath
# * @param  data   배열   데이터배열 
#
drawOpenSpending = (target, data)->
  x = d3.scale.ordinal().rangePoints([0, width], 0.3).domain(data.map (d)-> d.key)
  r = width / data.length / 2 * 0.9
  rscale = d3.scale.linear().range([0, r])
    .domain([0, Math.sqrt(d3.extent(data, (d)-> d["지출"])[1])])
  color = d3.scale.category20()
  currency = d3.format(',')

  svg = d3.select(target).append('svg')
    .attr('width', width + margin.width + 50).attr('height', height + margin.height)

  # 슬라이더영역
  slider = svg.append('g').attr
    height: 100
    width: width
    transform: "translate(#{margin.left + 50},#{margin.top})"
  # 메인영역 
  main = svg.append('g').attr
    width: width, height: height
    transform: "translate(#{margin.left + 50},#{margin.top + 100})"
  # 상세영역 
  detailmain = svg.append('g').attr
    width: width, height: height
    transform: "translate(#{margin.left},#{margin.top + r * 5 + 100})"

  # 메인이 될 원을 그린다
  node = main.selectAll('g.node').data(data).enter().append('g').classed('node', true)
    .attr('transform', (d)-> "translate(#{x(d.key)},#{r})")
  node.append('circle').attr('r', r).attr('fill', 'white')
    .attr('stroke', 'grey').attr('stroke-dasharray', '4,3')
  node.append('circle').attr('r', (d)-> rscale(Math.sqrt(d["지출액"] or 0)))
    .attr('fill', color(0))

  # 금액・용도 텍스트
  node.append('text').text((d)-> d.key)
    .style('text-anchor', 'middle').attr('dy', - r * 1.1)
  node.append('text').classed("spent", true).style('text-anchor', 'middle').attr('dy', r * 1.3)
    .text((d)-> "¥#{currency(d['지출액'] or 0)}")

  # 예산과의 차이
  main.append('text').text('지출액').attr(dx: -r * 2, dy: r * 2.5 + 8, 'font-size': '0.7em')
  main.append('text').text('예산현액').attr(dx: -r * 2, dy: r * 2.5 + 18, 'font-size': '0.7em')
  gap = node.append('g').classed('gap', true)
    .attr('transform', "translate(#{-r},#{r * 1.5})")
  gapBar = gap.selectAll('g').data((d)-> normalize(d3.entries(
    "지출액": d["지출액"]
    "예산현액": d["예산현액"]
    ))).enter()
  gapBar.append('rect').attr(
      y: (d, idx)-> 10 * idx
      width: (d)-> d.nVal * 2 * r or 0
      height: 10
      fill: (d)-> color(d.key)
    )
  gapBar.append('text').text((d)-> "¥#{currency(0|(d.value or 0) / 1000)}천엔") # 整数化
    .attr('dy', (d, idx)-> 10 * idx + 8).attr('font-size', '0.7em')

  # mouseover이벤트로 상세항목을 볼 수 있게 한다 
  detailx = d3.scale.ordinal().rangePoints([0, width], 0.3)
  node
    .on('mouseover', (d)->
      # 상세정보 배치 가져오기 
      detailx.domain(d.values.map (dd)-> dd["과목"])

      # circle의 fill을 리셋 
      main.selectAll('.node circle.out').attr('fill', color(0))
      # 새로 클릭된 것에 selected 클래스를 설정하고 색을 변경 
      d3.select(this).select('.out').attr('fill', 'white')

      # 새로 처리할 데이터 key 로서 과목을 설정 
      detail = detailmain.selectAll('g.detail').data(d.values, (dd)-> dd["과목"])

      # 삭제될 요소 
      detailExit = detail.exit().transition().duration(duration)
        .attr('transform', (dd)-> "translate(#{x(dd['구분'])},#{-r * 3})")
        .style('opacity', 0).remove()

      # 추가될 요소 
      detailEnter = detail.enter().append('g').classed('detail', true)
        .attr('transform', "translate(#{x(d.key)},#{-r * 1.5})")
        .style('opacity', 0)
      detailEnter.append('circle').attr('r', r * 1.0)
        .attr('fill', 'none').attr('stroke', 'grey').attr('stroke-dasharray', '4,3')
      detailEnter.append('circle').attr('r', (d)-> rscale(Math.sqrt(d["지출액"])))
        .attr('fill', color(0)).style('opacity', 0.8)
      detailEnter.append('text').text((dd)-> dd["과목"])
        .style('text-anchor', 'middle')
        .attr('dy', -r * 1.1)
      detailEnter.append('text').classed('spent', true).text((dd)-> "¥#{currency(dd['지출액'])}")
        .style('text-anchor', 'middle')
        .attr('dy',　r * 1.3)
      detailEnter.transition().duration(duration)
        .attr('transform', (dd)-> "translate(#{detailx(dd['과목'])},0)")
        .style('opacity', 1)
    )

  # 슬라이더 
  slider.append('text').classed('income', true).text("¥#{currency(1e6)}")
  slider.append('text').classed('spent', true).text("¥#{currency(0|1e6 * 0.06)}")
    .attr('transform', "translate(#{width * 0.9},0)")
  incomeRate = d3.scale.linear().domain([0, width * 0.6]).rangeRound([1e6, 2e7])
  drag = d3.behavior.drag()
    .on('drag', ()->
      point = d3.select(this)
      mv = +point.attr('cx') + d3.event.dx
      mv = Math.max(0, Math.min(mv, width * 0.6))
      point.attr('cx', mv)
      slider.select('.income').text("¥#{currency(incomeRate(mv))}")
      slider.select('.spent').text("¥#{currency(0|incomeRate(mv) * 0.06)}")
      main.selectAll('text.spent').text((d)->
        "¥#{currency(0|mv / (width * 0.6) * d['지출액'])}")
      detailmain.selectAll('text.spent').text((d)-> "¥#{currency(0|mv / (width * 0.6) * d['지출액'])}")
    )
  slideBar = slider.append('g')
    .attr('transform', "translate(#{width * 0.2},0)")
  slideBar.append('rect').attr(
    width: width * 0.6
    height: 10
    rx: 5
    fill: '#CCC'
  )
  slideBar.append('circle').attr(
    cx: 0
    cy: 5
    r: 10
    fill: '#CCC'
    stroke: 'black'
  ).style('cursor', 'move').call(drag)

#------------------------------------------------------------
#
# ## parseData
#
# 세입세출데이터를 parse하는 함수
#
# * @param  text       문자열     세입세출 텍스트 데이터
# * @return 오브젝트 세입세출별 JSON오브젝트배열의 오브젝트
#
parseData = (text)->
  typeStrs = text.split /\n\n/
  data = {}
  for typeStr in typeStrs
    [typeLabelStr, dataStr...] = typeStr.replace(/\n$/, '').split /\n/
    [label, unit] = typeStr.split /\s+/
    ssv = d3.dsv(" ", "text/ssv")
    if dataStr[dataStr.length - 1].match /^합계/
      dataStr.pop()
    data[label] = d3.nest().key((d)-> d["구분"] or "전체")
      .entries(ssv.parse dataStr.join("\n"))

    # 자릿수구분 콤마 삭제 
    for section in data[label]
      for item in section.values
        for k, v of item
          if v.match(/[\d\,]+/)
            item[k] = +v.replace(/,/g, '')
            section[k] = 0 unless section[k]
            section[k] += item[k]
            data[label][k] = 0 unless data[label][k]
            data[label][k] += item[k]
  data

#------------------------------------------------------------
#
# # 세입세출데이터를 가져와서 세금은 어디로 갔지 방식의 시각화를 작성한다
#
$.get("./kawasaki.dat").done (text)->
  data = parseData(text)
  drawOpenSpending "body", data["세출"]
