'use strict'
# 그리기 영역 정보 
margin = new Margin(100, 50)
[width, height] = [400, 400]

#------------------------------------------------------------
#
# ## drawBMI
#
# BMI의 꺾은선 그래프를 그린다
#
# * @param  target   문자열/selection SVG를 추가하는 요소
# * @param  dataurl  문자열           데이터의 URL
#
drawBMI = (target, dataurl)->
  color = d3.scale.category10()
  d3.csv dataurl, (data)->
    d3.select(target).append('h2').text("BMI의 꺾은선그래프")
    svg = d3.select(target).append('svg').attr(width: width + margin.width, height: height + margin.height)
    main = svg.append('g').attr(
      width: width, height: height
      transform: "translate(#{margin.left},#{margin.top})"
    )

    # 스케일
    x = d3.scale.ordinal().rangePoints([0, width])
      .domain(data.map (d)-> d["BMI"])
    y = d3.scale.linear().range([height, 0]).domain([0, d3.max(data, (d)-> +d["전체"])])

    # 경로 데이터 생성함수 
    line = d3.svg.line().x((d)-> x(d["BMI"])).y((d)-> y(+d["전체"]))

    # 축 
    main.append('g').call(d3.svg.axis().scale(x)
      .tickValues(x.domain().filter (d, idx)-> (idx % 2) is 0))
      .attr(transform: "translate(0,#{height})")
      .selectAll('path,line').attr(stroke: 'grey', fill: 'none')
    main.append('g').call(d3.svg.axis().scale(y).orient('left'))
      .selectAll('path,line').attr(stroke: 'grey', fill: 'none')

    # 라벨 
    main.append('text').text('나이')
      .attr(transform: "translate(#{width / 2},#{height + margin.bottom / 2})")
    main.append('text').text('BMI')
      .attr(transform: "translate(#{- margin.left / 2},#{height / 2}) rotate(-90)")

    main.append('path').datum(data).attr(
      d: line
      stroke: color(0)
      fill: 'none'
    )

# ------------------------------------------------------------
#
# ## drawBMIgender
#
# 남녀의 BMI 꺾은선그래프를 그린다
#
# * @param  target   문자열/selection SVG를 추가하는 요소 
# * @param  dataurl  문자열           데이터의 URL
#
drawBMIgender = (target, dataurl)->
  color = d3.scale.category10()
  d3.csv dataurl, (data)->
    d3.select(target).append('h2').text("남녀별 BMI의 꺾은선그래프")
    svg = d3.select(target).append('svg').attr(width: width + margin.width, height: height + margin.height)
    main = svg.append('g').attr(
      width: width, height: height
      transform: "translate(#{margin.left},#{margin.top})"
    )

    # 스케일 
    x = d3.scale.ordinal().rangePoints([0, width])
      .domain(data.map (d)-> d["BMI"])
    y = d3.scale.linear().range([height, 0]).domain([0, d3.max(data, (d)-> Math.max(+d["남성"], +d["여성"]))])

    # 경로 데이터 생성 함수 
    line = d3.svg.line().x((d)-> x(d["BMI"]))

    # 축 
    main.append('g').call(d3.svg.axis().scale(x)
      .tickValues(x.domain().filter (d, idx)-> (idx % 2) is 0))
      .attr(transform: "translate(0,#{height})")
      .selectAll('path,line').attr(stroke: 'grey', fill: 'none')
    main.append('g').call(d3.svg.axis().scale(y).orient('left'))
      .selectAll('path,line').attr(stroke: 'grey', fill: 'none')

    # 라벨 
    main.append('text').text('나이')
      .attr(transform: "translate(#{width / 2},#{height + margin.bottom / 2})")
    main.append('text').text('남녀별BMI')
      .attr(transform: "translate(#{- margin.left / 2},#{height / 2}) rotate(-90)")

    # 범례 
    legend = main.append('g').attr(transform: "translate(#{width - 50},#{50})")

    # 각각 그린다 
    for gender, idx in ["남성", "여성"]
      # 라인의 접근자를 변경 
      line.y((d)-> y(+d[gender]))
      main.append('path').datum(data).attr(
        d: line
        stroke: color(gender)
        fill: 'none'
      )
      # 범례에 추가 
      legend.append('text').text(gender).attr(
        fill: color(gender)
        stroke: 'none'
        dy: 16 * idx
      )


#------------------------------------------------------------
#
# ## drawBoxPlot
#
# 임금의 상자수염그림을 그린다 
#   10.10의 상자수염그림의 d3.svg.box을 이용한다 
#
# * @param  target   문자열/selection SVG를 추가하는 요소 
# * @param  dataurl  문자열           데이터의 URL
#
drawBoxPlot = (target, dataurl)->
  d3.csv dataurl, (data)->
    d3.select(target).append('h2').text("업계별 임금의 상자수염그림")
    # 데이터 정형 
    jobs = {}
    for d in data
      pref = d["지자체"]; delete d["지자체"]
      for job, salary of d
        jobs[job] = [] unless jobs[job]
        jobs[job].push {pref: pref, salary: salary}
    jobs = d3.entries(jobs)

    # SVG 요소를 작성 
    svg = d3.select(target).append('svg').attr(width: width + margin.width, height: height + margin.height)
    main = svg.append('g').attr(
      width: width, height: height
      transform: "translate(#{margin.left},#{margin.top})"
    )

    color = d3.scale.category10()

    # 스케일 작성 
    x = d3.scale.ordinal().domain(jobs.map (d)-> d.key).rangeBands([0, width], 0.5)
    y = d3.scale.linear().range([height, 0])
      .domain([0, d3.max(jobs, (d)-> d3.max(d.value, (dd)-> dd.salary))])

    # 축 
    xaxis = main.append('g').call(d3.svg.axis().scale(x))
      .attr(transform: "translate(0,#{height})")
    xaxis.selectAll('path,line').attr(stroke: 'grey', fill: 'none')
    xaxis.selectAll('text').attr(transform: "rotate(-10)").style('anchor-text': 'end')
    main.append('g').call(d3.svg.axis().scale(y).orient('left'))
      .selectAll('path,line').attr(stroke: 'grey', fill: 'none')

    # 라벨 
    main.append('text').text('업계')
      .attr(transform: "translate(#{width / 2},#{height + margin.bottom / 2})")
    main.append('text').text('임금')
      .attr(transform: "translate(#{- margin.left / 2},#{height / 2}) rotate(-90)")

    # 상자수염그림의 svg함수 
    box = d3.svg.box().values((d)-> d.value).value((d)-> d.salary).scale(y).width(x.rangeBand())
      .withHist(true)
    # 상자수염그림작성
    main.selectAll('g.box').data(jobs).enter().append('g').attr(
      class: 'box'
      transform: (d)-> "translate(#{x(d.key)},0)"
      stroke: (d)-> color(d.key)
    ).call(box)


#------------------------------------------------------------
#
# ## drawScatterMatrix
#
# 산포도행렬을 그린다 
#
# * @param  target   문자열/selection SVG를 추가하는 요소 
# * @param  dataurl  문자열           데이터의 URL
# * @param  opt      오브젝트       산포도행렬의 크기, 마진 설정 
#
drawScatterMatrix = (target, dataurl, opt)->
  format = (d)->
    prev = d["지자체"]
    delete d["지자체"]
    for k, v of d
      d[k] = +v
    {
      prev: prev
      data: d
    }
  d3.csv dataurl, format, (data)->
    d3.select(target).append('h2').text("산포도행렬")

    # 라벨을 가져온다 
    labels = Object.keys(data[0].data)

    # 각각의 산포도 위치 계산 
    W = d3.scale.ordinal().rangeBands([0, opt.width], .1).domain(labels)
    H = d3.scale.ordinal().rangeBands([opt.height, 0], .1).domain(labels)
    # 개개의 산포도 크기를 계산 
    w = W.rangeBand()
    h = W.rangeBand()

    # 범위를 가져온다 
    scales = {}
    for label in labels
      scales[label] =
        x: d3.scale.linear().range([0, w]).domain(d3.extent(data, (d)-> d.data[label]))
        y: d3.scale.linear().range([h, 0]).domain(d3.extent(data, (d)-> d.data[label]))
    color = d3.scale.category20()

    # svg 요소 작성 
    svg = d3.select('body').append('svg').attr(
      width: opt.width + opt.margin.width, height: opt.height + opt.margin.height)
    main = svg.append('g').attr(
      width: opt.width, height: opt.height, transform: "translate(#{opt.margin.left},#{opt.margin.top})")

    # 산포도 작성 
    for xLabel, xid in labels
      for yLabel, yid in labels
        cell = main.append('g').attr(
          width: w, height: h, transform: "translate(#{W(xLabel)},#{H(yLabel)})")
        cell.append('rect').attr(
          width: w, height: h, fill: 'none', stroke: 'grey')

        if xid isnt yid
          cell.selectAll('circle').data(data).enter().append('circle').attr(
            cx: (d)-> scales[xLabel].x(d.data[xLabel])
            cy: (d)-> scales[yLabel].y(d.data[yLabel])
            r: 2
            fill: (d)-> color(d.prev)
          )
        else
          # 대각요소 
          for k, idx in ['mean', 'median']
            cell.append('text').text(k + ": " + d3[k](data, (d)-> d.data[xLabel]).toFixed(0)).attr(
              dy: (idx + 1) * 12
              dx: w
            ).style('text-anchor': 'end', 'font-size': 10)

        # 산포도의 장식(라벨)
        # 좌단  
        if xid is 0
          cell.append('g').call(d3.svg.axis().orient('left').scale(scales[yLabel].y).ticks(4))
            .selectAll('path,line').attr(fill: 'none', stroke: 'grey')
          cell.append('text').text(yLabel).attr(transform: "translate(-60,#{h / 2}) rotate(-90)")
            .style('text-anchor': 'middle')
        # 상단  
        if yid is labels.length - 1
          xaxis = cell.append('g').call(d3.svg.axis().orient('top').scale(scales[xLabel].x).ticks(4))
          xaxis.selectAll('path,line').attr(fill: 'none', stroke: 'grey')
          xaxis.selectAll('text')
            .attr(transform: "rotate(90)", dx: -10, dy: 16).style('text-anchor': 'end')
          cell.append('text').text(xLabel).attr(transform: "translate(#{w / 2},-60)")
            .style('text-anchor': 'middle')

#------------------------------------------------------------
#
# # 시각화 
#
drawBMI('body', "./bmi.csv")
drawBMIgender('body', "./bmi.csv")
drawBoxPlot('body', "./tingin.csv")
drawScatterMatrix('body', "cancer.csv", {width: 800, height: 800, margin: new Margin(100,0,0,100)})
